/*
  ReadAnalogVoltage
  Reads an analog input on pin 0, converts it to voltage, and prints the result to the serial monitor.
  Attach the center pin of a potentiometer to pin A0, and the outside pins to +5V and ground.
 
 This example code is in the public domain.
 */
 
#include <Adafruit_CharacterOLED.h>
//#include <avr/eeprom.h>
#include <EEPROM.h>

#define int_mode RISING
#define DEBUG
// initialize the library with the OLED hardware 
// version OLED_Vx and numbers of the interface pins. 
// OLED_V1 = older, OLED_V2 = newer. If 2 doesn't work try 1 ;)
Adafruit_CharacterOLED lcd(OLED_V2, 6, 7, 8, 9, 10, 11, 12);

int ch=0;

long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
     ADMUX = _BV(MUX5) | _BV(MUX0) ;
  #else
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #endif  
 
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring
 
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH  
  uint8_t high = ADCH; // unlocks both
 
  long result = (high<<8) | low;
 
  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  return result-50; // Vcc in millivolts
}
float vref = 0, ftmp;
long vref2 = 0, tmp=0;
// the setup routine runs once when you press reset: 
byte bat0[8] = {
  0b01110,
  0b11111,
  0b10001,
  0b10001,
  0b10001,
  0b10001,
  0b11111,
  0b00000
};
byte bat1[8] = {
  0b01110,
  0b11111,
  0b10001,
  0b10001,
  0b10001,
  0b11111,
  0b11111,
  0b00000
};
byte bat2[8] = {
  0b01110,
  0b11111,
  0b10001,
  0b10001,
  0b11111,
  0b11111,
  0b11111,
  0b00000
};
byte bat3[8] = {
  0b01110,
  0b11111,
  0b10001,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b00000
};
byte bat4[8] = {
  0b01110,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b00000
};
void disp_ch(float ch, float vbat)
{int ch_n=map(ch, 0, 100, 0, 5);;
	
	switch ( ch_n )
      {
         case 0:
		    lcd.print ( char(1));
            break;
         case 1:
		    lcd.print ( char(2));
            break;
         case 2:
		    lcd.print ( char(3));
            break;
         case 3:
		    lcd.print ( char(4));
            break;
         case 4:
		    lcd.print ( char(5));
            break;
         default:
		    lcd.print ( char(5));
      }
}

uint32_t sp_stat, sp_stop, sp_time;
bool sp_ch=false, sp_st=false;

void speed_mt()
{
  detachInterrupt(0);
   /*if(sp_ch==true)
    {
      sp_stop = micros();
      sp_time=sp_stop-sp_start;
      sp_ch=false;
    }
   else if(sp_ch==false)
    {
      sp_start = micros();
      sp_ch=true;
    }*/
  sp_time=micros()-sp_stat;
  sp_stat = micros();
  attachInterrupt(0, speed_mt, int_mode);
}

struct settings_p
{
  double dist, av_speed, max_speed;
  int mode;
} settings;


void setup() {
  Serial.begin(9600);
  pinMode(4, OUTPUT);
  digitalWrite(4, LOW);
  delay(2000);
  Serial.println(readVcc()/ 1000.0);
  if((readVcc()/ 1000.0)<=4.5) {
      digitalWrite(4, LOW);
      EEPROM.write(0, 21);
      while(1){
       delay(1000);
       if(readVcc()/ 1000.0>4.5) {break;}
      }
    }
  digitalWrite(4, HIGH);
  pinMode(2, INPUT);    
  lcd.begin(20, 2);
  lcd.createChar (1, bat0);
  lcd.createChar (2, bat1);
  lcd.createChar (3, bat2);
  lcd.createChar (4, bat3);
  lcd.createChar (5, bat4);
  // initialize serial communication at 9600 bits per second:
  //attachInterrupt(0, speedm, int_mode);
  tmp=EEPROM.read(0);
  if(tmp!=21)
  {
    lcd.setCursor(0, 0);
    lcd.print("Can't read settings ");
    lcd.setCursor(0, 1);
    lcd.print("set all to default  ");
    delay(1000);
    lcd.clear();
  }
  if(tmp==21)
  {
    lcd.setCursor(0, 0);
    lcd.print("Starting parsing set");
    lcd.setCursor(0, 1);
    lcd.print("Load:              ");
    lcd.setCursor(7, 1);
    lcd.print("Distance     ");
    lcd.clear();
  }
  attachInterrupt(0, speed_mt, int_mode);
  delay(500);
}

int i = 0, fl = 0;
float voltage =0;
long sensorValue, sensorValue0;


void loop() {
// read the input on analog pin 0:
//  lcd.setCursor(16, 0);
//  lcd.print(sp_ch);
//  lcd.setCursor(18, 0);
//  lcd.print(digitalRead(2));
      
  vref = readVcc();
    if(vref <= 2820)
    {
      lcd.noDisplay();
      while(vref < 2820)
      {
        vref = readVcc();
        delay(2000);
      }
    }
    if(vref <= 3700 && fl==0)
    {
      lcd.setCursor(0, 0);
      lcd.print("Supply voltage low! ");
      lcd.setCursor(0, 1);
      lcd.print("Saving settings...  ");
      EEPROM.write(0, 21);
      //EEPROM.write(1, );
      //EEPROM.write(2, val);
      detachInterrupt(0);
      fl=1;
    }
    if(vref > 3700)
    {
      lcd.display();
      if(fl == 1)
      {
       lcd.clear();
       fl=0; 
       attachInterrupt(0, speed_mt, int_mode);
      }
    }
  sensorValue = analogRead(A1);

  //detachInterrupt(0);
  voltage = sensorValue * (vref / 1023.0);
  if(voltage<=3.0) {
    digitalWrite(4, LOW);}
  if(voltage>3.0) {
    digitalWrite(4, HIGH);}
  Serial.print("VCC:");
  Serial.print(vref);
  Serial.print(" ");
  Serial.print(" A1:");
  Serial.print(voltage);
  Serial.print(" SP:");
  Serial.println(sp_time);
  
  lcd.setCursor(0, 0);
  lcd.print("RT:");
  lcd.print(sp_time/1000000.0,2);
  lcd.print(" ");
  /*lcd.print(" LI:");
  lcd.print(sensorValue0/20);*/
  ftmp = ((voltage-2.8)/1.2)*100;
  if(ftmp<0){
  ftmp=0;}
  if(ftmp>100){
  ftmp=100;}
  lcd.setCursor(19, 0);
  disp_ch(ftmp,voltage);
  /*lcd.print(" CH:");
  lcd.print(((vref-2.7)/1.5)*100);
  lcd.print("%");*/
  lcd.setCursor(0, 1);
  lcd.print("SP:");
  if(sp_time>0) {
  ftmp=(192500000.0/sp_time)*0.036;}
  else {
  ftmp=0;}
  if((micros()-sp_stat)>=77000000) {
  ftmp=0;}
  lcd.print(ftmp);
  lcd.print("Km/H  ");
  delay(300);
  i=0;
  
}
